@@ -109,21 +109,32 @@ extern int rand();
 
 #endif /*SYSV or AMIGA*/
 
-#if (defined(MSDOS) || defined(AMIGA) )
 #include <fcntl.h>
 #include <time.h>
 #include <stdlib.h>
-#else
-#ifndef VMS
 #include <unistd.h>
-#endif
+/* 
+   Before Netpbm 9.0, atoi() and exit() were declared for everybody
+   except MSDOS and AMIGA, and time() and write() were declared for
+   everybody except MSDOS, AMIGA, and __osf__.  fcntl.h, time.h, and
+   stlib.h were included for MSDOS and AMIGA, and unistd.h was included
+   for everyone except VMS, MSDOS, and AMIGA.  With the netbsd patches,
+   atoi(), exit(), time(), and write() were not declared for __NetBSD__.
+
+   We're hoping that all current systems have the standard header
+   files, and will reinstate some of these explicit declarations if we
+   hear otherwise.  
+
+   If it turns out to be this easy, we should just move these inclusions
+   to the source files that actually need them.
+   
+   -Bryan 2000.04.13
+
 extern int atoi();
 extern void exit();
-#ifndef __osf__
 extern long time();
-extern int write();
-#endif
-#endif
+extern int write(); 
+*/
 
 /* CONFIGURE: On most BSD systems, malloc() gets declared in stdlib.h, on
 ** system V, it gets declared in malloc.h. On some systems, malloc.h
@@ -264,14 +275,10 @@ int pm_writelittleshort ARGS(( FILE* out, short s ));
 int pm_readlittlelong ARGS(( FILE* in, long* lP ));
 int pm_writelittlelong ARGS(( FILE* out, long l ));
 
-
-/* Command line option parsing */
-#include "pbmplus.h"
-/* Shhopt is Sverre Huseby's command line parsing package, which makes
-   writing command parsing code quick and error free.  Some of the Netpbm
-   programs use it.
+/* By making this <> instead of "", we avoid making shhopt.h a dependency
+   of every program in the package when we do make dep.
 */
-#include "shhopt.h"
+#include <shhopt.h>
 
 void pm_optParseOptions(int *argc, char *argv[],
                         optStruct opt[], int allowNegNum);
