@@ -20,9 +20,20 @@
     See if additional decompressor options effects signficant speedup.
     grayscale output of color image, downscaling, color quantization, and
     dithering are possibilities.  Djpeg's man page says they make it faster.
-    
-    Do wide pnms.
 
+  IMPLEMENTATION NOTE - PRECISION
+
+    There are two versions of the JPEG library.  One handles only JPEG
+    files with 8 bit samples; the other handles only 12 bit files.
+    This program may be compiled and linked against either, and run
+    dynamically linked to either at runtime independently.  It uses
+    the precisions information from the file header.  Note that when
+    the input has 12 bit precision, this program generates PPM files
+    with two-byte samples, but when the input has 8 bit precision, it
+    generates PPM files with one-byte samples.  One should use
+    Ppmdepth to reduce precision to 8 bits if one-byte-sample output
+    is essential.
+    
 *****************************************************************************/
 
 #include <ctype.h>		/* to declare isprint() */
@@ -32,6 +43,7 @@
 #include <errno.h>
 #include <jpeglib.h>
 #include "pnm.h"
+#include "shhopt.h"
 
 #define EXIT_WARNING 2  /* Goes with EXIT_FAILURE, EXIT_SUCCESS in stdlib.h */
 
@@ -118,7 +130,7 @@ parse_command_line(const int argc, char ** argv,
     argc_parse = argc;
     for (i=0; i < argc; i++) argv_parse[i] = argv[i];
 
-    optParseOptions(&argc_parse, argv_parse, option_def, 0);
+    pm_optParseOptions(&argc_parse, argv_parse, option_def, 0);
     /* Uses and sets argc_parse, argv_parse and all of *cmdline_p. */
 
     if (argc_parse - 1 == 0)
@@ -222,59 +234,17 @@ print_text_marker (j_decompress_ptr cinfo)
 
 
 
-
-/*
- * For 12-bit JPEG data, we either downscale the values to 8 bits
- * (to write standard byte-per-sample PPM/PGM files), or output
- * nonstandard word-per-sample PPM/PGM files.  Downscaling is done
- * if PPM_NORAWWORD is defined (this can be done in the Makefile
- * or in jconfig.h).
- * (When the core library supports data precision reduction, a cleaner
- * implementation will be to ask for that instead.)
- */
-
-/* SAMPLE_SQUEEZE is a macro that takes a JSAMPLE (sample read from a JPEG
-   file) and generates a 'pixval' (sample in the pnm output buffer), even
-   though the JPEG file may have a wider sample than the pnm format allows.
-   */
-
-#if BITS_IN_JSAMPLE == 8
-#define PUTPPMSAMPLE(ptr,v)  *ptr++ = (char) (v)
-#define SAMPLE_SQUEEZE(x) (x)
-#define BYTESPERSAMPLE 1
-#define PPM_MAXVAL 255
-#else
-#ifdef PPM_NORAWWORD
-#define PUTPPMSAMPLE(ptr,v)  *ptr++ = (char) ((v) >> (BITS_IN_JSAMPLE-8))
-#define SAMPLE_SQUEEZE(x) ((x) >> (BITS_IN_JSAMPLE-8))
-#define BYTESPERSAMPLE 1
-#define PPM_MAXVAL 255
-#else
-/* The word-per-sample format always puts the LSB first. */
-#define PUTPPMSAMPLE(ptr,v)			\
-	{ register int val_ = v;		\
-	  *ptr++ = (char) (val_ & 0xFF);	\
-	  *ptr++ = (char) ((val_ >> 8) & 0xFF);	\
-	}
-#ifdef BIGGRAYS
-#define SAMPLE_SQUEEZE(x) (x)
-#else
-#define SAMPLE_SQUEEZE(x) ((x) >> (BITS_IN_JSAMPLE-8))
-#endif
-#define BYTESPERSAMPLE 2
-#define PPM_MAXVAL ((1<<BITS_IN_JSAMPLE)-1)
-#endif
-#endif
-
-
-typedef struct rgb {int r; int g; int b;} rgb_type;
+typedef struct rgb {unsigned int r; unsigned int g; unsigned int b;} rgb_type;
 
 
 static rgb_type *
-read_rgb(JSAMPLE *ptr, J_COLOR_SPACE color_space) {
+read_rgb(JSAMPLE *ptr, J_COLOR_SPACE color_space, const unsigned int maxval) {
 /*----------------------------------------------------------------------------
   Return the RGB triple corresponding to the color of the JPEG pixel at
-  'ptr', which is in color space 'color_space';
+  'ptr', which is in color space 'color_space'.  
+
+  Assume 'maxval' is the maximum sample value in the input pixel, and also
+  use it for the maximum sample value in the return value.
 -----------------------------------------------------------------------------*/
     static rgb_type rgb;  /* Our return value */
 
@@ -291,9 +261,9 @@ read_rgb(JSAMPLE *ptr, J_COLOR_SPACE color_space) {
         const int y = GETJSAMPLE(*(ptr+2));
         const int k = GETJSAMPLE(*(ptr+3));
 
-        rgb.r = ((MAXJSAMPLE-k)*(MAXJSAMPLE-c))/MAXJSAMPLE;
-        rgb.g = ((MAXJSAMPLE-k)*(MAXJSAMPLE-y))/MAXJSAMPLE;
-        rgb.b = ((MAXJSAMPLE-k)*(MAXJSAMPLE-m))/MAXJSAMPLE;
+        rgb.r = ((maxval-k)*(maxval-c))/maxval;
+        rgb.g = ((maxval-k)*(maxval-y))/maxval;
+        rgb.b = ((maxval-k)*(maxval-m))/maxval;
     }
         break;
     default:
@@ -317,6 +287,7 @@ static void
 copy_pixel_row(const JSAMPROW jpegbuffer, const int width, 
                const unsigned int samples_per_pixel, 
                const J_COLOR_SPACE color_space,
+               const unsigned int maxval,
                FILE * const output_file, const int output_type) {
   JSAMPLE *ptr;
   unsigned int output_cursor;     /* Cursor into output buffer 'pnmbuffer' */
@@ -326,20 +297,16 @@ copy_pixel_row(const JSAMPROW jpegbuffer, const int width,
   for (output_cursor = 0; output_cursor < width; output_cursor++) {
       xel current_pixel;
       if (samples_per_pixel >= 3) {
-          const rgb_type * const rgb_p = read_rgb(ptr, color_space);
-          PPM_ASSIGN(current_pixel, 
-                     SAMPLE_SQUEEZE(rgb_p->r),
-                     SAMPLE_SQUEEZE(rgb_p->g),
-                     SAMPLE_SQUEEZE(rgb_p->b)
-                     );
+          const rgb_type * const rgb_p = read_rgb(ptr, color_space, maxval);
+          PPM_ASSIGN(current_pixel, rgb_p->r, rgb_p->g, rgb_p->b);
       } else {
-          PNM_ASSIGN1(current_pixel, SAMPLE_SQUEEZE(GETJSAMPLE(*ptr)));
+          PNM_ASSIGN1(current_pixel, GETJSAMPLE(*ptr));
       }
       ptr += samples_per_pixel;  /* move to next pixel of input */
       pnmbuffer[output_cursor] = current_pixel;
   }
   pnm_writepnmrow(output_file, pnmbuffer, width,
-                  PPM_MAXVAL, output_type, FALSE);
+                  maxval, output_type, FALSE);
 }
 
 
@@ -395,42 +362,101 @@ set_color_spaces(const J_COLOR_SPACE jpeg_color_space,
 
 
 
+static void
+setup_jpeg_input(FILE *input_file, 
+                 struct jpeg_decompress_struct * const cinfo_p,
+                 struct jpeg_error_mgr * const jerr_p, 
+                 const boolean verbose, 
+                 const int trace_level, 
+                 const J_DCT_METHOD dct_method, 
+                 const int max_memory_to_use, 
+                 const boolean nosmooth) {
+/*----------------------------------------------------------------------------
+   Open the input file, read the header, create decompressor object (and
+   allocate memory for it), set up decompressor.
+-----------------------------------------------------------------------------*/
+    /* Initialize the JPEG decompression object with default error handling. */
+    cinfo_p->err = jpeg_std_error(jerr_p);
+    jpeg_create_decompress(cinfo_p);
+    if (trace_level == 0 && verbose) cinfo_p->err->trace_level = 1;
+    else cinfo_p->err->trace_level = trace_level;
+
+    /* Insert custom marker processor for COM and APP12.
+     * APP12 is used by some digital camera makers for textual info,
+     * so we provide the ability to display it as text.
+     * If you like, additional APPn marker types can be selected for display,
+     * but don't try to override APP0 or APP14 this way (see libjpeg.doc).
+     */
+    jpeg_set_marker_processor(cinfo_p, JPEG_COM, print_text_marker);
+    jpeg_set_marker_processor(cinfo_p, JPEG_APP0+12, print_text_marker);
+
+    /* Specify data source for decompression */
+    jpeg_stdio_src(cinfo_p, input_file);
+
+    /* Read file header, set default decompression parameters */
+    (void) jpeg_read_header(cinfo_p, TRUE);
+
+    cinfo_p->dct_method = dct_method;
+    if (max_memory_to_use != -1)
+        cinfo_p->mem->max_memory_to_use = max_memory_to_use;
+    if (nosmooth)
+        cinfo_p->do_fancy_upsampling = FALSE;
+
+}
+
+
+
+static void 
+cleanup(FILE* input_file, FILE* output_file, xel *pnmbuffer, 
+        struct jpeg_decompress_struct *cinfo_p) {
+/*----------------------------------------------------------------------------
+   Release memory, close out everything.
+-----------------------------------------------------------------------------*/
+    int rc;
+
+    pnm_freerow(pnmbuffer);
+
+    rc = fclose(output_file);
+    if (rc == EOF) 
+        pm_error("Error writing output file.  Errno = %s (%d).",
+                 strerror(errno), errno);
+    
+    /* Finish decompression and release decompressor memory. */
+    (void) jpeg_finish_decompress(cinfo_p);
+    jpeg_destroy_decompress(cinfo_p);
+
+    /* Close files, if we opened them */
+    if (input_file != stdin) fclose(input_file);
+
+    /* Program may have exited with non-zero completion code vie
+       various function calls above.  
+    */
+}
+
+
+
 int
 main(int argc, char **argv) {
   struct jpeg_decompress_struct cinfo;
   struct jpeg_error_mgr jerr;
   FILE * input_file;
   FILE * output_file;
-  int rc;
   int output_type;
     /* The type of output file, PGM or PPM.  Value is either PPM_TYPE
        or PGM_TYPE, which conveniently also pass as format values
        PPM_FORMAT and PGM_FORMAT.
     */
   JSAMPROW jpegbuffer;  /* Input buffer.  Filled by jpeg_scanlines() */
-
+  unsigned int maxval;  
+    /* The maximum value of a sample (color component), both in the input
+       and the output.
+       */
 
   pnm_init(&argc, argv);
-
   progname = argv[0];
 
   parse_command_line(argc, argv, &cmdline);
 
-  /* Initialize the JPEG decompression object with default error handling. */
-  cinfo.err = jpeg_std_error(&jerr);
-  jpeg_create_decompress(&cinfo);
-  if (cmdline.trace_level == 0 && cmdline.verbose) cinfo.err->trace_level = 1;
-  else cinfo.err->trace_level = cmdline.trace_level;
-
-  /* Insert custom marker processor for COM and APP12.
-   * APP12 is used by some digital camera makers for textual info,
-   * so we provide the ability to display it as text.
-   * If you like, additional APPn marker types can be selected for display,
-   * but don't try to override APP0 or APP14 this way (see libjpeg.doc).
-   */
-  jpeg_set_marker_processor(&cinfo, JPEG_COM, print_text_marker);
-  jpeg_set_marker_processor(&cinfo, JPEG_APP0+12, print_text_marker);
-
   /* Open the input file */
   if (cmdline.input_filename == NULL) 
       input_file = stdin;
@@ -443,27 +469,22 @@ main(int argc, char **argv) {
 
   output_file = stdout;
 
-  /* Specify data source for decompression */
-  jpeg_stdio_src(&cinfo, input_file);
-
-  /* Read file header, set default decompression parameters */
-  (void) jpeg_read_header(&cinfo, TRUE);
-
-  cinfo.dct_method = cmdline.dct_method;
-  if (cmdline.max_memory_to_use != -1)
-      cinfo.mem->max_memory_to_use = cmdline.max_memory_to_use;
-  if (cmdline.nosmooth)
-      cinfo.do_fancy_upsampling = FALSE;
-
+  setup_jpeg_input(input_file, &cinfo, &jerr, cmdline.verbose, 
+                   cmdline.trace_level, cmdline.dct_method, 
+                   cmdline.max_memory_to_use, cmdline.nosmooth);
+                   
   set_color_spaces(cinfo.jpeg_color_space, &output_type, 
                    &cinfo.out_color_space);
+  
+  maxval = (1 << cinfo.data_precision) - 1;
 
-  if (cmdline.verbose) 
+  if (cmdline.verbose) {
+      pm_message("Input image data precision = %d bits", cinfo.data_precision);
       pm_message("Output file will have format %c%c "
                  "with max sample value of %d.", 
                  (char) (output_type/256), (char) (output_type % 256),
-                 PPM_MAXVAL);
-  
+                 maxval);
+  }  
   /* Calculate output image dimensions so we can allocate space */
   jpeg_calc_output_dimensions(&cinfo);
 
@@ -477,7 +498,7 @@ main(int argc, char **argv) {
 
   /* Write pnm output header */
   pnm_writepnminit(output_file, cinfo.output_width, cinfo.output_height,
-                   PPM_MAXVAL, output_type, FALSE);
+                   maxval, output_type, FALSE);
 
   pnmbuffer = pnm_allocrow(cinfo.output_width);
 
@@ -485,25 +506,16 @@ main(int argc, char **argv) {
   while (cinfo.output_scanline < cinfo.output_height) {
     jpeg_read_scanlines(&cinfo, &jpegbuffer, 1);
     copy_pixel_row(jpegbuffer, cinfo.output_width, cinfo.out_color_components,
-                   cinfo.out_color_space, output_file, output_type);
+                   cinfo.out_color_space, maxval, output_file, output_type);
   }
 
-  pnm_freerow(pnmbuffer);
-
-  rc = fclose(output_file);
-  if (rc == EOF) 
-      pm_error("Error writing output file.  Errno = %s (%d).",
-               strerror(errno), errno);
-
-  /* Finish decompression and release decompressor memory. */
-  (void) jpeg_finish_decompress(&cinfo);
-  jpeg_destroy_decompress(&cinfo);
-
-  /* Close files, if we opened them */
-  if (input_file != stdin) fclose(input_file);
-
-  /* Program may have exited with non-zero completion code vie various function
-     calls above.
-     */
+  cleanup(input_file, output_file, pnmbuffer, &cinfo); 
+  
   exit(jerr.num_warnings > 0 ? EXIT_WARNING : EXIT_SUCCESS);
+
 }
+
+
+
+
+
