@@ -162,11 +162,8 @@ getinit( file, colsP, rowsP, padrightP, maxvalP, visualclassP, formatP, colorsP
     X10WDFileHeader* h10P;
     X11WDFileHeader* h11P;
 
-    byte_swap = 0;
-    if ( PNM_MAXMAXVAL < 65535L )
-	*maxvalP = PNM_MAXMAXVAL;
-    else
-	*maxvalP = 65535L;
+    byte_swap = 0;       /* Initial assumption */
+	*maxvalP = 65535L;   /* Initial assumption */
     h10P = (X10WDFileHeader*) header;
     h11P = (X11WDFileHeader*) header;
     if ( sizeof(*h10P) > sizeof(*h11P) )
