@@ -1484,15 +1484,10 @@ int mode;
 		}
 #endif /* AMIGA or VMS */
 
-/* This should really be PPM_MAXMAXVAL, but that can be big, and then
- * I'd have to conditionally output raw/not-raw PPM, which is a pain.
- */
-#define MY_MAXVAL (255)
-
 		fprintf(pnmscale, "P6\n%d %d\n%d\n",
-			rectwidth(&clipsrc), rectheight(&clipsrc), MY_MAXVAL);
+			rectwidth(&clipsrc), rectheight(&clipsrc), PPM_MAXMAXVAL);
 
-#define REDEPTH(c, oldmax)  ((c) * ((MY_MAXVAL) + 1) / (oldmax + 1))
+#define REDEPTH(c, oldmax)  ((c) * ((PPM_MAXMAXVAL) + 1) / (oldmax + 1))
 
 		switch (pixSize) {
 		case 8:
